#! /usr/bin/python

from gi.repository import Gtk
import gettext
import os
import dbus
import fcntl

gettext.install("cinnamon", "/usr/share/cinnamon/locale")

MIRRORED_ICON	= "/usr/lib/cinnamon-multimonitor-switcher/icons/mirror.png"
DOCKED_ICON	= "/usr/lib/cinnamon-multimonitor-switcher/icons/dock.png"
EXTENDED_ICON	= "/usr/lib/cinnamon-multimonitor-switcher/icons/extension.png"
INTERNAL_ICON	= "/usr/lib/cinnamon-multimonitor-switcher/icons/internal.png"

# Singleton
fh = 0
def run_once():
	global fh

	fh = open(os.path.realpath(__file__), 'r')

	try:
		fcntl.flock(fh, fcntl.LOCK_EX|fcntl.LOCK_NB)
	except:
		os._exit(0)

class CMSWindow(Gtk.Window):
	def __init__(self):
		Gtk.Window.__init__(self, title=_("Cinnamon Multi-Monitor Switcher"))
# Main window conf
		self.set_border_width(10)
		self.set_position(Gtk.WindowPosition.CENTER)
		self.set_default_size(350, 500)
		self.set_resizable(False)
		self.set_decorated(False)
		self.set_opacity(0.8)
		self.set_icon_name("backgrounds")

# dbus init
		bus = dbus.SessionBus()
		bus_obj = bus.get_object('org.gnome.SettingsDaemon', '/org/cinnamon/SettingsDaemon/XRANDR')
		self.iface = dbus.Interface(bus_obj, 'org.cinnamon.SettingsDaemon.XRANDR_2')

# drawing GUI stuff
		hbox = Gtk.Box(spacing=6)
		self.add(hbox)

# Internal mode button
		i_button = Gtk.Button()
		i_button.connect("clicked", self.on_internal_button_clicked)
		i_image = Gtk.Image()
                i_image.set_from_file(INTERNAL_ICON)
		i_button.add(i_image)

		hbox.pack_start(i_button, True, True, 0)

# Mirrored mode button
		m_button = Gtk.Button()
		m_button.connect("clicked", self.on_mirrored_button_clicked)
		m_image = Gtk.Image()
		m_image.set_from_file(MIRRORED_ICON)
		m_button.add(m_image)

		hbox.pack_start(m_button, True, True, 0)

# Docked monitor mode button
		d_button = Gtk.Button()
		d_button.connect("clicked", self.on_docked_button_clicked)
		d_image = Gtk.Image()
		d_image.set_from_file(DOCKED_ICON)
		d_button.add(d_image)

		hbox.pack_start(d_button, True, True, 0)

# extended mode button
		e_button = Gtk.Button()
		e_button.connect("clicked", self.on_extended_button_clicked)
		e_image = Gtk.Image()
		e_image.set_from_file(EXTENDED_ICON)
		e_button.add(e_image)

		hbox.pack_start(e_button, True, True, 0)

	def get_connectors(self):
		vga_output = os.popen('xrandr | grep VGA | grep " connected " | awk \'{print $1}\'')
                vga = vga_output.read().strip('\n')

                lvds_output = os.popen('xrandr | grep LVDS | grep " connected " | awk \'{print $1}\'')
                lvds = lvds_output.read().strip('\n')

		return (vga, lvds)

	def on_mirrored_button_clicked(self, widget):
		vga, lvds = self.get_connectors()

		if (lvds and vga):
# avoids VideoModeSwitch() display mode switch in turn issue.
			os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
			self.iface.VideoModeSwitch(1)
			print ("mirrored mode")
			Gtk.main_quit()
		else:
			print("nothing... mirrored mode button pressed..")
			Gtk.main_quit()

	def on_internal_button_clicked(self, widget):
		vga, lvds = self.get_connectors()
		if (lvds and vga):
			os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
			print("internal mode")
			Gtk.main_quit()
		else:
			print("nothing... internal mode button pressed..")
			Gtk.main_quit()

	def on_docked_button_clicked(self, widget):
		vga, lvds = self.get_connectors()
		if (lvds and vga):
			os.system("xrandr --output %s --auto --output %s --off" % (vga, lvds))
			print("external mode")
			Gtk.main_quit()
		else:
			print("nothing... docked mode button pressed..")
			Gtk.main_quit()

	def on_extended_button_clicked(self, widget):
		vga, lvds = self.get_connectors()
		if (lvds and vga):
			os.system("xrandr --output %s --off --output %s --auto" % (vga, lvds))
			os.system("xrandr --output %s --auto --output %s --auto --right-of %s" % (lvds, vga, lvds))
			print("extended mode.")
			Gtk.main_quit()
		else:
			print("nothing... extended mode button pressed..")
			Gtk.main_quit()

run_once()

win = CMSWindow()
win.connect("delete-event", Gtk.main_quit)
win.show_all()
Gtk.main()

